<?php
/**
 * @version		1.2.7
 * @package		Joomla
 * @subpackage	EShop - Fedex Shipping
 * @author  	Giang Dinh Truong
 * @copyright	Copyright (C) 2012 Ossolution Team
 * @license		GNU/GPL, see LICENSE.php
 */
// no direct access
defined( '_JEXEC' ) or die();

class eshop_fedex extends eshop_shipping
{

	/**
	 *
	 * Constructor function
	 */
	function eshop_fedex()
	{
		parent::setName('eshop_fedex');
		parent::eshop_shipping();
	}
	
	/**
	 * 
	 * Function tet get quote for price shipping
	 * @param array $addressData
	 * @param object $params
	 */
	function getQuote($addressData, $params)
	{
		//Check geozone condition
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		if (!$params->get('geozone_id'))
		{
			$status = true;
		}
		else
		{
			$query->select('COUNT(*)')
				->from('#__eshop_geozonezones')
				->where('geozone_id = ' . intval($params->get('geozone_id')))
				->where('country_id = ' . intval($addressData['country_id']))
				->where('(zone_id = 0 OR zone_id = ' . intval($addressData['zone_id']) . ')');
			$db->setQuery($query);
			if ($db->loadResult())
			{
				$status = true;
			}
			else
			{
				$status = false;
			}
		}
		//Check min sub total for free condition
		$cart = new EshopCart();
		$subTotal = $cart->getSubTotal();
		$minTotalForFree = $params->get('min_sub_total_for_free', 0);
		if ($minTotalForFree > 0 && $minTotalForFree <= $subTotal)
		{
			$status = false;
		}
		//Check input data condition
		$subTotalRange = $params->get('sub_total_range');
		$subTotalRangeArr = explode(';', $subTotalRange);
		$costRange = $params->get('cost_range');
		$costRangeArr = explode(';', $costRange);
		if (!count($subTotalRangeArr) || !count($costRangeArr))
		{
			$status = false;
		}
		$methodData = array();
		if ($status)
		{
			$cart = new EshopCart();
			$currency = new EshopCurrency();
			$tax = new EshopTax(EshopHelper::getConfig());
			$eshopWeight = new EshopWeight();
			$eshopLength = new EshopLength();
			$cartWeight = $cart->getWeight();

			// Get weight and weight code
			$weight = $eshopWeight->convert($cartWeight, EshopHelper::getConfigValue('weight_id'), $params->get('fedex_weight_id'));
			$weight = ($weight < 0.1 ? 0.1 : $weight);
			$weightCode = strtoupper($eshopWeight->getUnit($params->get('fedex_weight_id')));
			
			$date = time();
			$day = date('l', $date);
			if ($day == 'Saturday')
				$date += 172800;
			elseif ($day == 'Sunday')
				$date += 86400;
			
			$countryInfo = EshopHelper::getCountry(EshopHelper::getConfigValue('country_id'));
			$zoneInfo = EshopHelper::getZone(EshopHelper::getConfigValue('zone_id'));
			
			if (!$params->get('fedex_test'))
				$url = 'https://gateway.fedex.com/web-services/';
			else
				$url = 'https://gatewaybeta.fedex.com/web-services/';

			// Whoever introduced xml to shipping companies should be flogged
			$xml  = '<?xml version="1.0"?>';
			$xml .= '<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://fedex.com/ws/rate/v10">';
			$xml .= '	<SOAP-ENV:Body>';
			$xml .= '		<ns1:RateRequest>';
			$xml .= '			<ns1:WebAuthenticationDetail>';
			$xml .= '				<ns1:UserCredential>';
			$xml .= '					<ns1:Key>' . $params->get('fedex_key') . '</ns1:Key>';
			$xml .= '					<ns1:Password>' . $params->get('fedex_password') . '</ns1:Password>';
			$xml .= '				</ns1:UserCredential>';
			$xml .= '			</ns1:WebAuthenticationDetail>';
			$xml .= '			<ns1:ClientDetail>';
			$xml .= '				<ns1:AccountNumber>' . $params->get('fedex_account') . '</ns1:AccountNumber>';
			$xml .= '				<ns1:MeterNumber>' . $params->get('fedex_meter') . '</ns1:MeterNumber>';
			$xml .= '			</ns1:ClientDetail>';
			$xml .= '			<ns1:Version>';
			$xml .= '				<ns1:ServiceId>crs</ns1:ServiceId>';
			$xml .= '				<ns1:Major>10</ns1:Major>';
			$xml .= '				<ns1:Intermediate>0</ns1:Intermediate>';
			$xml .= '				<ns1:Minor>0</ns1:Minor>';
			$xml .= '			</ns1:Version>';
			$xml .= '			<ns1:ReturnTransitAndCommit>true</ns1:ReturnTransitAndCommit>';
			$xml .= '			<ns1:RequestedShipment>';
			$xml .= '				<ns1:ShipTimestamp>' . date('c', $date) . '</ns1:ShipTimestamp>';
			$xml .= '				<ns1:DropoffType>' . $params->get('fedex_dropoff_type') . '</ns1:DropoffType>';
			$xml .= '				<ns1:PackagingType>' . $params->get('fedex_packaging_type') . '</ns1:PackagingType>';
			$xml .= '				<ns1:Shipper>';
			$xml .= '					<ns1:Contact>';
			$xml .= '						<ns1:PersonName>' . EshopHelper::getConfigValue('store_owner') . '</ns1:PersonName>';
			$xml .= '						<ns1:CompanyName>' . EshopHelper::getConfigValue('store_name') . '</ns1:CompanyName>';
			$xml .= '						<ns1:PhoneNumber>' . EshopHelper::getConfigValue('telephone') . '</ns1:PhoneNumber>';
			$xml .= '					</ns1:Contact>';
			$xml .= '					<ns1:Address>';

			if ($countryInfo->iso_code_2 == 'US' || $countryInfo->iso_code_2 == 'CA')
				$xml .= '						<ns1:StateOrProvinceCode>' . ($zoneInfo ? $zoneInfo->zone_code : '') . '</ns1:StateOrProvinceCode>';
			else
				$xml .= '						<ns1:StateOrProvinceCode></ns1:StateOrProvinceCode>';
				
			$xml .= '						<ns1:PostalCode>' . $params->get('fedex_postcode') . '</ns1:PostalCode>';
			$xml .= '						<ns1:CountryCode>' . $countryInfo->iso_code_2 . '</ns1:CountryCode>';
			$xml .= '					</ns1:Address>';
			$xml .= '				</ns1:Shipper>';
			
			$xml .= '				<ns1:Recipient>';
			$xml .= '					<ns1:Contact>';
			$xml .= '						<ns1:PersonName>' . $addressData['firstname'] . ' ' . $addressData['lastname'] . '</ns1:PersonName>';
			$xml .= '						<ns1:CompanyName>' . $addressData['company'] . '</ns1:CompanyName>';
			$xml .= '						<ns1:PhoneNumber></ns1:PhoneNumber>';
			$xml .= '					</ns1:Contact>';
			$xml .= '					<ns1:Address>';
			$xml .= '						<ns1:StreetLines>' . $addressData['address_1'] . '</ns1:StreetLines>';
			$xml .= '						<ns1:City>' . $addressData['city'] . '</ns1:City>';
			
			if ($addressData['iso_code_2'] == 'US' || $addressData['iso_code_2'] == 'CA')
				$xml .= '						<ns1:StateOrProvinceCode>' . $addressData['zone_code'] . '</ns1:StateOrProvinceCode>';
			else
				$xml .= '						<ns1:StateOrProvinceCode></ns1:StateOrProvinceCode>';
			
			$xml .= '						<ns1:PostalCode>' . $addressData['postcode'] . '</ns1:PostalCode>';
			$xml .= '						<ns1:CountryCode>' . $addressData['iso_code_2'] . '</ns1:CountryCode>';
			$xml .= '						<ns1:Residential>' . ($addressData['company'] ? 'true' : 'false') . '</ns1:Residential>';
			$xml .= '					</ns1:Address>';
			$xml .= '				</ns1:Recipient>';
			$xml .= '				<ns1:ShippingChargesPayment>';
			$xml .= '					<ns1:PaymentType>SENDER</ns1:PaymentType>';
			$xml .= '					<ns1:Payor>';
			$xml .= '						<ns1:AccountNumber>' . $params->get('fedex_account') . '</ns1:AccountNumber>';
			$xml .= '						<ns1:CountryCode>' . $countryInfo->iso_code_2 . '</ns1:CountryCode>';
			$xml .= '					</ns1:Payor>';
			$xml .= '				</ns1:ShippingChargesPayment>';
			$xml .= '				<ns1:RateRequestTypes>' . $params->get('fedex_rate_type') . '</ns1:RateRequestTypes>';
			$xml .= '				<ns1:PackageCount>1</ns1:PackageCount>';
			$xml .= '				<ns1:RequestedPackageLineItems>';
			$xml .= '					<ns1:SequenceNumber>1</ns1:SequenceNumber>';
			$xml .= '					<ns1:GroupPackageCount>1</ns1:GroupPackageCount>';
			$xml .= '					<ns1:Weight>';
			$xml .= '						<ns1:Units>' . $weightCode . '</ns1:Units>';
			$xml .= '						<ns1:Value>' . $weight . '</ns1:Value>';
			$xml .= '					</ns1:Weight>';
			$xml .= '					<ns1:Dimensions>';
			$xml .= '						<ns1:Length>10</ns1:Length>';
			$xml .= '						<ns1:Width>10</ns1:Width>';
			$xml .= '						<ns1:Height>3</ns1:Height>';
			$xml .= '						<ns1:Units>IN</ns1:Units>';
			$xml .= '					</ns1:Dimensions>';
			$xml .= '				</ns1:RequestedPackageLineItems>';
			$xml .= '			</ns1:RequestedShipment>';
			$xml .= '		</ns1:RateRequest>';
			$xml .= '	</SOAP-ENV:Body>';
			$xml .= '</SOAP-ENV:Envelope>';
			
			$curl = curl_init($url);
			
			curl_setopt($curl, CURLOPT_POST, true);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $xml);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_HEADER, false);
			curl_setopt($curl, CURLOPT_TIMEOUT, 30);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
				
			$response = curl_exec($curl);
			curl_close($curl);
				
			$dom = new DOMDocument('1.0', 'UTF-8');
			$dom->loadXml($response);
			
			$error = '';
			$quoteData = array();
			if ($dom->getElementsByTagName('HighestSeverity')->item(0)->nodeValue == 'FAILURE' || $dom->getElementsByTagName('HighestSeverity')->item(0)->nodeValue == 'ERROR' || $dom->getElementsByTagName('HighestSeverity')->item(0)->nodeValue == 'WARNING')
			{
				$error = $dom->getElementsByTagName('HighestSeverity')->item(0)->nodeValue;
			}
			else
			{
				$rateReplyDetails = $dom->getElementsByTagName('RateReplyDetails');
				foreach ($rateReplyDetails as $rateReplyDetail)
				{
					$code = strtolower($rateReplyDetail->getElementsByTagName('ServiceType')->item(0)->nodeValue);
					if (in_array(strtoupper($code), $params->get('fedex_service')))
					{
						$title = JText::_('PLG_ESHOP_FEDEX_' . strtoupper($code));
						if ($params->get('fedex_display_time'))
						{
							$title .= ' (' . JText::_('PLG_ESHOP_FEDEX_ETA') . ' ' . date(JText::_('PLG_ESHOP_FEDEX_DATE_FORMAT') . ' ' . JText::_('PLG_ESHOP_FEDEX_TIME_FORMAT'), strtotime($rateReplyDetail->getElementsByTagName('DeliveryTimestamp')->item(0)->nodeValue)) . ')';
						}
						$totalNetCharge = $rateReplyDetail->getElementsByTagName('RatedShipmentDetails')->item(0)->getElementsByTagName('ShipmentRateDetail')->item(0)->getElementsByTagName('TotalNetCharge')->item(0);
						$cost = $totalNetCharge->getElementsByTagName('Amount')->item(0)->nodeValue;
						$currencyCode = $totalNetCharge->getElementsByTagName('Currency')->item(0)->nodeValue;
						$quoteData[$code] = array(
								'name'			=> 'eshop_fedex.' . $code,
								'title'			=> $title,
								'cost'			=> $currency->convert($cost, $currencyCode, EshopHelper::getConfigValue('default_currency_code')),
								'taxclass_id'	=> $params->get('fedex_taxclass_id'),
								'text'			=> $currency->format($tax->calculate($currency->convert($cost, $currencyCode, $currency->getCurrencyCode()), $params->get('fedex_taxclass_id'), EshopHelper::getConfigValue('tax')), $currency->getCurrencyCode(), 1.0000000)
						);
					}
				}
			}
			$title = JText::_('PLG_ESHOP_FEDEX_TITLE');
			
			if ($params->get('fedex_display_weight'))
			{
				$title .= ' (' . JText::_('PLG_ESHOP_FEDEX_WEIGHT') . ' ' . $eshopWeight->format($weight, $params->get('fedex_weight_id')) . ')';
			}
				
			$query->clear();
			$query->select('*')
				->from('#__eshop_shippings')
				->where('name = "eshop_fedex"');
			$db->setQuery($query);
			$row = $db->loadObject();
				
			$methodData = array(
					'name'			=> 'eshop_fedex',
					'title'			=> $title,
					'quote'			=> $quoteData,
					'ordering'		=> $row->ordering,
					'error'			=> $error
			);
		}
		return $methodData;
	}
}